#
# After check_pendings() has cleaned up after a stuck reINVITE, a destroy is scheduled,
# but the still outstanding reINVITE blocks the destruction. This prevents that from
# happening.
#

Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 369601)
+++ channels/chan_sip.c	(working copy)
@@ -3850,7 +3850,7 @@
 			ast_debug(3, "Re-scheduled destruction of SIP call %s\n", p->callid ? p->callid : "<unknown>");
 			append_history(p, "ReliableXmit", "timeout");
 			if (sscanf(p->lastmsg, "Tx: %30s", method_str) == 1 || sscanf(p->lastmsg, "Rx: %30s", method_str) == 1) {
-				if (method_match(SIP_CANCEL, method_str) || method_match(SIP_BYE, method_str)) {
+				if (p->ongoing_reinvite || method_match(SIP_CANCEL, method_str) || method_match(SIP_BYE, method_str)) {
 					pvt_set_needdestroy(p, "autodestruct");
 				}
 			}

#
# Clear reinviteid before calling check_pendings() so that check_pendings knows
# it is safe to proceed.
#
Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 369601)
+++ channels/chan_sip.c	(working copy)
@@ -6229,8 +6229,8 @@
 {
 	struct sip_pvt *dialog = (struct sip_pvt *) data;
 	struct ast_channel *owner = sip_pvt_lock_full(dialog);
+	dialog->reinviteid = -1;
 	check_pendings(dialog);
-	dialog->reinviteid = -1;
 	if (owner) {
 		ast_channel_unlock(owner);
 		ast_channel_unref(owner);

#
# With a broken UA, or "attack", even if we are still waiting for a reINVITE response,
# check_pendings() was sending an instant BYE. This also happened if a good UA responded
# with 100 Trying.
#
# This reserves the check_pending until the reinviteid timer has expired, or is cleared by
# a 200 packet or higher.
#
Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 369601)
+++ channels/chan_sip.c	(working copy)
@@ -19864,8 +19864,11 @@
 static void check_pendings(struct sip_pvt *p)
 {
 	if (ast_test_flag(&p->flags[0], SIP_PENDINGBYE)) {
-		/* if we can't BYE, then this is really a pending CANCEL */
-		if (p->invitestate == INV_PROCEEDING || p->invitestate == INV_EARLY_MEDIA) {
+		if (p->reinviteid > -1) {
+			/* Outstanding p->reinviteid timeout, so wait... */
+			return;
+		} else if (p->invitestate == INV_PROCEEDING || p->invitestate == INV_EARLY_MEDIA) {
+			/* if we can't BYE, then this is really a pending CANCEL */
 			p->invitestate = INV_CANCELLED;
 			transmit_request(p, SIP_CANCEL, p->lastinvite, XMIT_RELIABLE, FALSE);
 			/* If the cancel occurred on an initial invite, cancel the pending BYE */
